import datetime
import json
from io import BytesIO

from django.contrib.auth.models import User
from django.db.models import Q
from django.test import (
    RequestFactory,
    TestCase,
)
from django.urls import (
    reverse,
    reverse_lazy,
)
from django.utils import timezone as tz
from django.utils.translation import ugettext, ugettext_lazy
from pypdf import PdfReader

from souschef.meal.factories import (
    ComponentFactory,
    ComponentIngredientFactory,
    IncompatibilityFactory,
    IngredientFactory,
    RestrictedItemFactory,
)
from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES_SIDES,
    Component,
    Component_ingredient,
    Ingredient,
    Menu,
)
from souschef.member.factories import (
    AddressFactory,
    ClientFactory,
    DeliveryHistoryFactory,
    MemberFactory,
    RouteFactory,
)
from souschef.member.models import (
    DAYS_OF_WEEK,
    Client,
    Client_avoid_ingredient,
    DeliveryHistory,
    Member,
    Restriction,
    Route,
)
from souschef.order.models import Order
from souschef.sous_chef.tests import TestMixin as SousChefTestMixin

from .filters import KitchenCountOrderFilter


class KitchenCountReportTestCase(SousChefTestMixin, TestCase):
    # This data set includes 'Ground porc' clashing ingredient
    # This data set includes 'Tracy' client lastname
    #   and his side dish is 'compote'
    fixtures = ["sample_data"]

    def setUp(self):
        self.force_login()

    def test_clashing_ingredient(self):
        """An ingredient we know will clash must be in the page"""
        today = datetime.date.today()
        # create orders today
        clients = Client.active.all()
        Order.objects.auto_create_orders(today, clients)
        # main dish and its ingredients today
        main_dishes = Component.objects.filter(name="Ginger pork")
        main_dish = main_dishes[0]
        dish_ingredients = Component.get_recipe_ingredients(main_dish.id)
        for ing in dish_ingredients:
            ci = Component_ingredient(component=main_dish, ingredient=ing, date=today)
            ci.save()
        # Add extra ingredient so that more clashing clients will
        #  require two pages on the PDF kitchen count report
        extra = Ingredient.objects.get(name="Walnuts")
        ci = Component_ingredient(component=main_dish, ingredient=extra, date=today)
        ci.save()
        # add sides ingredient
        sides_component = Component.objects.get(
            component_group=COMPONENT_GROUP_CHOICES_SIDES
        )
        extra = Ingredient.objects.get(name="Cabbage")
        ci = Component_ingredient(
            component=sides_component, ingredient=extra, date=today
        )
        ci.save()
        # menu today
        Menu.create_menu_and_components(
            today,
            [
                "Ginger pork",
                "Green Salad",
                "Fruit Salad",
                "Day s Dessert",
                "Day s Diabetic Dessert",
                "Day s Pudding",
                "Day s Compote",
            ],
        )
        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {"delivery_date": datetime.date.today(), "download": "kitchen_count"},
        )
        self.assertEqual(response.status_code, 200)
        pdf_data = PdfReader(BytesIO(response.content))
        pdf_content = pdf_data.pages[0].extract_text()
        self.assertTrue("Ground porc" in pdf_content)

    def test_no_components(self):
        """No orders on this day therefore no documents to download."""
        menu_day = datetime.date(1999, 1, 1)
        Menu.create_menu_and_components(
            menu_day,
            [
                "Ginger pork",
                "Green Salad",
                "Fruit Salad",
                "Day s Dessert",
                "Day s Diabetic Dessert",
                "Day s Pudding",
                "Day s Compote",
            ],
        )
        # main dish and its ingredients today
        main_dishes = Component.objects.filter(name="Ginger pork")
        main_dish = main_dishes[0]
        dish_ingredients = Component.get_recipe_ingredients(main_dish.id)
        for ing in dish_ingredients:
            ci = Component_ingredient(
                component=main_dish, ingredient=ing, date=menu_day
            )
            ci.save()
        # add sides ingredient
        sides_component = Component.objects.get(
            component_group=COMPONENT_GROUP_CHOICES_SIDES
        )
        extra = Ingredient.objects.get(name="Cabbage")
        ci = Component_ingredient(
            component=sides_component, ingredient=extra, date=menu_day
        )
        ci.save()

        response = self.client.get(
            reverse("delivery:kitchen_count"),
            {"delivery_date": menu_day.isoformat()},
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue(b"No kitchen count report available" in response.content)

    def test_pdf_kitchen_count_and_labels_reports(self):
        # generate orders today
        today = datetime.date.today()
        clients = Client.active.all()
        Order.objects.auto_create_orders(today, clients)
        # main dish and its ingredients today
        main_dishes = Component.objects.filter(name="Ginger pork")
        main_dish = main_dishes[0]
        dish_ingredients = Component.get_recipe_ingredients(main_dish.id)
        for ing in dish_ingredients:
            ci = Component_ingredient(component=main_dish, ingredient=ing, date=today)
            ci.save()
        # Add sides ingredient
        sides_component = Component.objects.get(
            component_group=COMPONENT_GROUP_CHOICES_SIDES
        )
        # This ingredient is in the restrictions of some clients in the data
        sides_ingredient = Ingredient.objects.get(name="Brussel sprouts")
        ci = Component_ingredient(
            component=sides_component, ingredient=sides_ingredient, date=today
        )
        ci.save()
        # menu today
        Menu.create_menu_and_components(
            today,
            [
                "Ginger pork",
                "Green Salad",
                "Fruit Salad",
                "Day s Dessert",
                "Day s Diabetic Dessert",
                "Day s Pudding",
                "Day s Compote",
                "Days Sides",
            ],
        )

        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {
                "delivery_date": datetime.date.today().isoformat(),
                "download": "kitchen_count",
            },
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue("ReportLab" in repr(response.content))

        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {"delivery_date": datetime.date.today().isoformat(), "download": "labels"},
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue("ReportLab" in repr(response.content))

    def test_ingredients_not_defined_redirect(self):
        # generate orders
        today = datetime.date.today()
        clients = Client.active.all()
        created_orders = Order.objects.auto_create_orders(today, clients)
        self.assertTrue(created_orders)

        num_menu_comp_created = Menu.create_menu_and_components(
            today,
            [
                "Ginger pork",
                "Green Salad",
                "Fruit Salad",
                "Day s Dessert",
                "Day s Diabetic Dessert",
                "Day s Pudding",
                "Day s Compote",
            ],
        )
        self.assertEqual(num_menu_comp_created, 7)

        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {
                "delivery_date": today.isoformat(),
            },
        )
        # Ingredients were not defined; kitchen count page will redirect to the
        # ingredients page.
        self.assertEqual(response.status_code, 302)


class ChooseDayMainDishIngredientsTestCase(SousChefTestMixin, TestCase):
    fixtures = ["sample_data"]

    @classmethod
    def setUpTestData(cls):
        # This data set includes 'Ginger pork' main dish
        # This data set includes 'Coq au vin' main dish
        # This data set includes 'Ground porc' main dish ingredient
        # This data set includes 'Pepper' available ingredient
        # create orders for today
        clients = Client.active.all()
        Order.objects.auto_create_orders(datetime.date.today(), clients)

    def setUp(self):
        self.force_login()

    def test_known_ingredients(self):
        """Two ingredients we know must be in the page"""
        response = self.client.get(
            reverse_lazy("delivery:meal"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertTrue(
            b"Ground porc" in response.content and b"Pepper" in response.content
        )

    def test_date_with_dish_next(self):
        """From ingredient choice go to Kitchen Count Report."""
        response = self.client.get(reverse_lazy("delivery:meal"))
        maindish = Component.objects.get(name="Ginger pork")
        cis = Component_ingredient.objects.filter(date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]
        req = {}
        req["_update"] = "Confirm all ingredients"
        req["maindish"] = str(maindish.id)
        req["ingredients"] = ing_ids
        req["sides_ingredients"] = [
            Ingredient.objects.filter(name="zucchini").first().id
        ]
        req["delivery_date"] = datetime.date.today().isoformat()
        response = self.client.post(reverse_lazy("delivery:meal"), req)
        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {
                "delivery_date": datetime.date.today().isoformat(),
                "download": "kitchen_count",
            },
        )
        self.assertEqual(response.status_code, 200)
        pdf_data = PdfReader(BytesIO(response.content))
        pdf_content = pdf_data.pages[0].extract_text()
        self.assertTrue("Ginger pork" in pdf_content)

    def test_remember_day_ingredients(self):
        """After Kitchen Count Report we remember chosen ingredients."""
        response = self.client.get(reverse_lazy("delivery:meal"))
        maindish = Component.objects.get(name="Ginger pork")
        cis = Component_ingredient.objects.filter(date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]
        req = {}
        req["_update"] = "Confirm all ingredients"
        req["maindish"] = str(maindish.id)
        req["ingredients"] = ing_ids
        req["sides_ingredients"] = [Ingredient.objects.filter(name="Onions").first().id]
        req["delivery_date"] = datetime.date.today().isoformat()
        response = self.client.post(reverse_lazy("delivery:meal"), req)
        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {
                "delivery_date": datetime.date.today().isoformat(),
                "download": "kitchen_count",
            },
        )
        self.assertEqual(response.status_code, 200)
        pdf_data = PdfReader(BytesIO(response.content))
        pdf_content = pdf_data.pages[0].extract_text()
        self.assertTrue("Ginger pork" in pdf_content)
        response = self.client.get(
            reverse_lazy("delivery:meal"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertTrue(b"Ginger pork" in response.content)

    def test_restore_dish_recipe(self):
        """Restore dish ingredients to those of recipe."""
        # dish : Ginger pork with added Pepper
        response = self.client.get(reverse_lazy("delivery:meal"))
        maindish = Component.objects.get(name="Ginger pork")
        cis = Component_ingredient.objects.filter(date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]
        # dish and added ingredient must be in kitchen count
        req = {}
        req["_update"] = "Confirm all ingredients"
        req["maindish"] = str(maindish.id)
        req["ingredients"] = ing_ids + [Ingredient.objects.get(name="Pepper").id]
        req["sides_ingredients"] = [Ingredient.objects.all().first().id]
        req["delivery_date"] = datetime.date.today().isoformat()
        response = self.client.post(reverse_lazy("delivery:meal"), req)
        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {
                "delivery_date": datetime.date.today().isoformat(),
                "download": "kitchen_count",
            },
        )
        self.assertEqual(response.status_code, 200)
        pdf_data = PdfReader(BytesIO(response.content))
        pdf_content = pdf_data.pages[0].extract_text()
        self.assertTrue("Ginger pork" in pdf_content)
        self.assertTrue("Pepper" in pdf_content)
        # restore recipe
        response = self.client.post(
            reverse_lazy("delivery:meal"),
            {
                "_restore": "Restore recipe",
                "delivery_date": datetime.date.today().isoformat(),
            },
        )
        # update ingredients
        req = {}
        req["_update"] = "Confirm all ingredients"
        req["maindish"] = str(maindish.id)
        req["ingredients"] = ing_ids
        req["sides_ingredients"] = [Ingredient.objects.all().first().id]
        req["delivery_date"] = datetime.date.today().isoformat()
        response = self.client.post(reverse_lazy("delivery:meal"), req)
        # check that we have Ginger pork with no Pepper in Kitchen count
        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {
                "delivery_date": datetime.date.today().isoformat(),
                "download": "kitchen_count",
            },
        )
        self.assertEqual(response.status_code, 200)
        pdf_data = PdfReader(BytesIO(response.content))
        pdf_content = pdf_data.pages[0].extract_text()
        self.assertTrue("Ginger pork" in pdf_content)
        self.assertTrue("Pepper" not in pdf_content)

    def test_change_main_dish(self):
        """Change dish then go directly to Kitchen Count Report."""
        maindish = Component.objects.get(name="Coq au vin")
        cis = Component_ingredient.objects.filter(date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]

        response = self.client.get(reverse_lazy("delivery:meal_id", args=[maindish.id]))
        req = {}
        req["_update"] = "Update ingredients"
        req["maindish"] = str(maindish.id)
        req["ingredients"] = ing_ids
        req["sides_ingredients"] = [Ingredient.objects.all().first().id]
        req["delivery_date"] = datetime.date.today().isoformat()
        response = self.client.post(reverse_lazy("delivery:meal"), req)
        response = self.client.get(
            reverse_lazy("delivery:kitchen_count"),
            {
                "delivery_date": datetime.date.today().isoformat(),
                "download": "kitchen_count",
            },
        )
        self.assertEqual(response.status_code, 200)
        pdf_data = PdfReader(BytesIO(response.content))
        pdf_content = pdf_data.pages[0].extract_text()
        self.assertTrue("Coq au vin" in pdf_content)

    def test_post_invalid_form(self):
        """Invalid form."""
        response = self.client.get(reverse_lazy("delivery:meal"))
        req = {}
        req["_update"] = "Next: Print Kitchen Count"
        req["maindish"] = "wrong"
        req["delivery_date"] = datetime.date.today().isoformat()
        response = self.client.post(reverse_lazy("delivery:meal"), req)

        self.assertIn(
            ugettext(
                "Select a valid choice. That choice is not one of "
                "the available choices."
            ).encode(response.charset),
            response.content,
        )

    def test_if_valid_no_red_sign(self):
        o = Order.objects.first()
        o.delivery_date = tz.now()
        o.save()
        response = self.client.get(reverse("delivery:order"))
        self.assertNotIn(b'<i class="warning sign red icon"></i>', response.content)
        response = self.client.post(
            reverse("delivery:refresh_orders"),
            {"generateOrderDate": datetime.date.today().isoformat()},
        )
        self.assertNotIn(b'<i class="warning sign red icon"></i>', response.content)

    def test_no_route_error(self):
        o = Order.objects.first()
        o.delivery_date = tz.now()
        o.save()
        c = o.client
        c.route = None
        c.save()
        response = self.client.get(
            reverse("delivery:order"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertIn(b'<i class="warning sign red icon"></i>', response.content)
        response = self.client.post(
            reverse("delivery:refresh_orders"),
            {"generateOrderDate": datetime.date.today().isoformat()},
        )
        self.assertIn(b'<i class="warning sign red icon"></i>', response.content)

    def test_not_geolocalized_error(self):
        o = Order.objects.first()
        o.delivery_date = tz.now()
        o.save()
        c = o.client
        c.member.address.latitude = None
        c.member.address.longitude = None
        c.member.address.save()
        response = self.client.get(
            reverse("delivery:order"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertIn(b'<i class="warning sign red icon"></i>', response.content)
        response = self.client.post(
            reverse("delivery:refresh_orders"),
            {"generateOrderDate": datetime.date.today().isoformat()},
        )
        self.assertIn(b'<i class="warning sign red icon"></i>', response.content)

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:meal")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:meal")
        # Run
        response = self.client.get(
            url,
            {"delivery_date": datetime.date.today().isoformat()},
        )
        # Check
        self.assertEqual(response.status_code, 200)

    def test_disable_form_fields_for_users_that_do_not_have_edit_permission(
        self,
    ):  # noqa
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:meal")
        # Run
        response = self.client.get(
            url,
            {"delivery_date": datetime.date.today().isoformat()},
        )
        # Check
        for field in response.context["form"]:
            assert field.field.disabled

    def test_do_not_allow_users_that_do_not_have_edit_permission_to_post_data(
        self,
    ):  # noqa
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:meal")
        # Run
        response = self.client.post(
            url,
            {"delivery_date": datetime.date.today().isoformat()},
        )
        # Check
        self.assertEqual(response.status_code, 403)


class DeliveryRouteSheetTestCase(SousChefTestMixin, TestCase):
    # This data set includes 'Blondin' client lastname
    #   in 'Centre Sud' route
    # This data set includes 'Tracy' client lastname
    #   in 'Mile-End' route
    #   and his side dish is 'compote'
    fixtures = ["sample_data"]

    def setUp(self):
        # generate orders today
        self.today = datetime.date.today()
        clients = Client.active.all()
        Order.objects.auto_create_orders(self.today, clients)
        self.route_id = Route.objects.get(name="Centre Sud").id
        self.force_login()

    def test_query(self):
        """Sample route sheet query."""
        route_list = Order.get_delivery_list(self.today, self.route_id)
        self.assertTrue("Blondin" in repr(route_list))

    def test_extra_similar_sides(self):
        """Test cumulative quantities for similar side dishes."""
        # Add two separate extra compote order items for 'Tracy'
        member = Member.objects.filter(lastname="Tracy")[0]
        client = Client.objects.get(member=member.id)
        order = Order.objects.get(client=client.id, delivery_date=self.today)
        self.assertEqual(order.client.id, client.id)
        mile_end_id = Route.objects.get(name="Mile-End").id
        route_list = Order.get_delivery_list(self.today, mile_end_id)
        self.assertTrue("Tracy" in repr(route_list))

    def test_sheet(self):
        """Sample route sheet page."""
        client = Client.objects.filter(member__lastname="Blondin")[0]
        route = Route.objects.get(pk=self.route_id)
        DeliveryHistoryFactory(
            route=route, date=tz.datetime.today(), client_id_sequence=[client.id]
        )
        response = self.client.get(
            reverse_lazy("delivery:route_sheet", args=[self.route_id]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertTrue(b"Blondin" in response.content)

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:route_sheet", args=[self.route_id])
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        DeliveryHistoryFactory(
            route=Route.objects.get(pk=self.route_id), date=tz.datetime.today()
        )
        url = reverse("delivery:route_sheet", args=[self.route_id])
        # Run
        response = self.client.get(
            url,
            {"delivery_date": datetime.date.today().isoformat()},
        )
        # Check
        self.assertEqual(response.status_code, 200)


class RedirectAnonymousUserTestCase(SousChefTestMixin, TestCase):
    fixtures = ["sample_data"]

    def test_anonymous_user_gets_redirect_to_login_page(self):
        check = self.assertRedirectsWithAllMethods
        check(reverse("delivery:order"))
        check(reverse("delivery:meal"))
        meal = Component.objects.first()
        meal_id = meal.id
        check(reverse("delivery:meal_id", kwargs={"id": meal_id}))
        check(reverse("delivery:routes"))
        check(reverse("delivery:kitchen_count"))
        check(reverse("delivery:route_sheet", kwargs={"pk": 1}))
        check(reverse("delivery:refresh_orders"))


class OrderlistViewTestCase(SousChefTestMixin, TestCase):
    fixtures = ["sample_data"]

    def setUp(self):
        super().setUp()
        self.force_login()

    def test_can_filter_orders_by_client_names(self):
        # Setup
        Order.objects.all().update(delivery_date=tz.now())
        url = reverse("delivery:order")
        # Run
        response = self.client.get(url, {"client_name": "john"})
        # Check
        self.assertEqual(
            set(response.context["orders"]),
            set(
                Order.objects.filter(
                    Q(client__member__firstname__icontains="john")
                    | Q(client__member__lastname__icontains="john")
                )
            ),
        )

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:order")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:order")
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class KitchenCountOrderFilterTestCase(SousChefTestMixin, TestCase):
    fixtures = ["sample_data"]

    def setUp(self):
        super().setUp()
        self.factory = RequestFactory()

    def test_can_filter_orders_by_client_names(self):
        # Setup
        queryset = Order.objects.all()
        request_1 = self.factory.get("/")
        request_2 = self.factory.get("/", {"client_name": "john doe"})
        # Run
        filterset_1 = KitchenCountOrderFilter(request_1.GET, queryset)
        filterset_2 = KitchenCountOrderFilter(request_2.GET, queryset)
        # Check
        self.assertEqual(filterset_1.qs.count(), queryset.count())
        self.assertTrue(filterset_2.qs.count())
        self.assertEqual(
            set(filterset_2.qs),
            set(
                Order.objects.filter(
                    Q(client__member__firstname__icontains="john")
                    | Q(client__member__firstname__icontains="doe")
                    | Q(client__member__lastname__icontains="john")
                    | Q(client__member__lastname__icontains="doe")
                )
            ),
        )


class RoutesInformationViewTestCase(SousChefTestMixin, TestCase):
    fixtures = ["sample_data"]

    def setUp(self):
        super().setUp()
        self.force_login()

    def test_can_embed_additional_route_sheet_information_when_printing(self):
        # Setup
        url = reverse("delivery:routes")
        # Run
        response_1 = self.client.get(
            url,
            {"delivery_date": datetime.date.today().isoformat()},
        )
        response_2 = self.client.get(
            url,
            {"download": "yes", "delivery_date": datetime.date.today().isoformat()},
        )
        # Check
        self.assertNotIn("routes_dict", response_1)
        self.assertIn("routes_dict", response_2)

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:routes")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:routes")
        # Run
        response = self.client.get(
            url,
            {"delivery_date": datetime.date.today().isoformat()},
        )
        # Check
        self.assertEqual(response.status_code, 200)


class KitchenCountViewTestCase(SousChefTestMixin, TestCase):
    fixtures = ["sample_data"]

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:kitchen_count")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:kitchen_count")

        # main dish and its ingredients today
        main_dishes = Component.objects.filter(name="Ginger pork")
        main_dish = main_dishes[0]
        dish_ingredients = Component.get_recipe_ingredients(main_dish.id)
        for ing in dish_ingredients:
            ci = Component_ingredient(
                component=main_dish, ingredient=ing, date=datetime.date.today()
            )
            ci.save()
        # Add sides ingredient
        sides_component = Component.objects.get(
            component_group=COMPONENT_GROUP_CHOICES_SIDES
        )
        sides_ingredient = Ingredient.objects.get(name="Brussel sprouts")
        ci = Component_ingredient(
            component=sides_component,
            ingredient=sides_ingredient,
            date=datetime.date.today(),
        )
        ci.save()

        # Run
        response = self.client.get(
            url,
            {"delivery_date": datetime.date.today().isoformat()},
        )
        # Check
        self.assertEqual(response.status_code, 200)


class RefreshOrderViewTestCase(SousChefTestMixin, TestCase):
    def test_redirects_users_who_do_not_have_edit_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:refresh_orders")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_edit_permission(self):
        # Setup
        user = User.objects.create_superuser(
            username="foo", email="foo@example.com", password="secure"
        )
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("delivery:refresh_orders")
        # Run
        response = self.client.post(
            url,
            {"generateOrderDate": datetime.date.today().isoformat()},
        )
        # Check
        self.assertEqual(response.status_code, 200)

    def test_refresh_create_only_if_scheduled_today(self):
        """
        Refs bug #734.
        If a client is ongoing but don't have scheduled delivery,
        then don't create his order.
        """
        meals_default = {}
        data = {
            "main_dish_{}_quantity": 2,
            "size_{}": "R",
            "dessert_{}_quantity": 0,
            "diabetic_dessert_{}_quantity": 0,
            "fruit_salad_{}_quantity": 1,
            "green_salad_{}_quantity": 1,
            "pudding_{}_quantity": 1,
            "compote_{}_quantity": 0,
        }
        for day, _ in DAYS_OF_WEEK:
            for key_tmpl, value in data.items():
                meals_default[key_tmpl.format(day)] = value

        ongoing_clients = ClientFactory.create_batch(
            10,
            status=Client.ACTIVE,
            delivery_type=Client.ONGOING_DELIVERY,
            meal_default_week=meals_default,
            route=RouteFactory(),
        )

        # The meal schedule should exclude today.
        schedule_delivery = []
        weekday = datetime.date.today().weekday()
        for i, t in enumerate(DAYS_OF_WEEK):
            if i != weekday:
                schedule_delivery.append(t[0])

        for c in ongoing_clients:
            c.set_simple_meals_schedule(schedule_delivery)

        self.force_login()
        url = reverse("delivery:refresh_orders")
        response = self.client.post(
            url, {"generateOrderDate": datetime.date.today().isoformat()}
        )
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Order.objects.all().count(), 0)  # No order should be created.


class ExcludeMisconfiguredClientsTestCase(SousChefTestMixin, TestCase):
    """
    Test 4 clients:
    - c_valid: correctly configured
               id==10, avoid veggie_10->chicken_10, avoid wine_10
    - c_nr:    misconfigured: route==None
               id==20, avoid veggie_20->chicken_20, avoid wine_20
    - c_ng:    misconfigured: not .isgeolocalized
               id==30, avoid veggie_30->chicken_30, avoid wine_30
    - c_nrng:  misconfigured: route==None and not .isgeolocalized
               id==40, avoid veggie_40->chicken_40, avoid wine_40

    Test meal = chicken_10 + wine_10 + chicken_20 + wine_20 + ...

    c_nr, c_ng and c_nrng should be excluded.
    """

    @classmethod
    def setUpTestData(cls):
        meal_default_week = {
            "main_dish_friday_quantity": 1,
            "main_dish_monday_quantity": 1,
            "main_dish_saturday_quantity": 1,
            "main_dish_sunday_quantity": 1,
            "main_dish_thursday_quantity": 1,
            "main_dish_tuesday_quantity": 1,
            "main_dish_wednesday_quantity": 1,
            "size_friday": "L",
            "size_monday": "L",
            "size_saturday": "L",
            "size_sunday": "L",
            "size_thursday": "L",
            "size_tuesday": "L",
            "size_wednesday": "L",
            "dessert_friday_quantity": 1,
            "dessert_monday_quantity": 1,
            "dessert_saturday_quantity": 1,
            "dessert_sunday_quantity": 1,
            "dessert_thursday_quantity": 1,
            "dessert_tuesday_quantity": 1,
            "dessert_wednesday_quantity": 1,
        }
        cls.route1 = RouteFactory()
        cls.route2 = RouteFactory()
        cls.c_valid = ClientFactory(
            pk=10,
            status=Client.ACTIVE,
            delivery_type=Client.ONGOING_DELIVERY,  # ongoing
            route=cls.route1,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Valid", lastname="Valid", address=AddressFactory()
            ),
        )
        cls.c_nr = ClientFactory(
            pk=20,
            status=Client.ACTIVE,
            delivery_type=Client.ONGOING_DELIVERY,  # ongoing
            route=None,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Nronly", lastname="Nronly", address=AddressFactory()
            ),
        )
        cls.c_ng = ClientFactory(
            pk=30,
            status=Client.ACTIVE,
            delivery_type=Client.ONGOING_DELIVERY,  # ongoing
            route=cls.route2,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Ngonly",
                lastname="Ngonly",
                address=AddressFactory(latitude=None),
            ),
        )
        cls.c_nrng = ClientFactory(
            pk=40,
            status=Client.ACTIVE,
            delivery_type=Client.ONGOING_DELIVERY,  # ongoing
            route=None,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Nrng",
                lastname="Nrng",
                address=AddressFactory(longitude=None),
            ),
        )

        cls.main_dish = ComponentFactory(name="coq au vin", component_group="main_dish")
        cls.sides = ComponentFactory(name="sides", component_group="sides")

        cls.ingred_chickens = []
        cls.ingred_wines = []
        for client in (cls.c_valid, cls.c_nr, cls.c_ng, cls.c_nrng):
            # Ensure that orders are created when refrshing orders.
            client.set_simple_meals_schedule(
                [
                    "monday",
                    "tuesday",
                    "wednesday",
                    "thursday",
                    "friday",
                    "saturday",
                    "sunday",
                ]
            )

            # client-specific ingredient (for testing clashes)
            this_chicken = IngredientFactory(
                name=f"chicken_{client.id}", ingredient_group="meat"
            )
            this_wine = IngredientFactory(
                name=f"wine_{client.id}", ingredient_group="oils_and_sauces"
            )

            # add them to main_dish
            ComponentIngredientFactory(component=cls.main_dish, ingredient=this_chicken)
            ComponentIngredientFactory(component=cls.main_dish, ingredient=this_wine)

            # all special veggies, special winehaters
            this_incompatib = IncompatibilityFactory(
                restricted_item=RestrictedItemFactory(
                    name=f"veggie_{client.id}", restricted_item_group="meat"
                ),
                ingredient=this_chicken,
            )
            Restriction.objects.create(
                client=client, restricted_item=this_incompatib.restricted_item
            )
            Client_avoid_ingredient.objects.create(client=client, ingredient=this_wine)

            # keep track of these chickens and wines
            cls.ingred_chickens.append(this_chicken)
            cls.ingred_wines.append(this_wine)

    def setUp(self):
        self.force_login()

    def _refresh_orders(self):
        return self.client.post(
            reverse("delivery:refresh_orders"),
            {"generateOrderDate": datetime.date.today().isoformat()},
        )

    def _today_meal(self):
        return self.client.post(
            reverse("delivery:meal"),
            {
                "maindish": [str(self.main_dish.id)],
                "_update": ["Next: Print Kitchen Count"],
                "ingredients": map(
                    lambda ingred: str(ingred.id),
                    self.ingred_chickens + self.ingred_wines,
                ),
                "sides_ingredients": map(
                    lambda ingred: str(ingred.id), self.ingred_chickens
                ),
                "delivery_date": datetime.date.today().isoformat(),
            },
        )

    def test_step_1__review_orders(self):
        response = self.client.get(
            reverse("delivery:order"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(len(response.context["orders"]), 0)
        self.assertEqual(
            response.content.count(b'<i class="warning sign red icon"></i>'), 0
        )
        # refresh to create orders
        response = self._refresh_orders()
        self.assertEqual(len(response.context["orders"]), 4)
        self.assertEqual(
            response.content.count(b'<i class="warning sign red icon"></i>'), 3
        )

    def test_step_3__component_table(self):
        _ = self._refresh_orders()
        response = self._today_meal()
        self.assertRedirects(
            response,
            reverse("delivery:meal")
            + f"?delivery_date={datetime.date.today().isoformat()}",
        )
        response = self.client.get(
            reverse("delivery:kitchen_count"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        component_lines = response.context["component_lines"]
        main_dish_component_line = next(
            cl
            for cl in component_lines
            if cl.component_group == ugettext_lazy("Main Dish")
        )
        self.assertEqual(main_dish_component_line.rqty, 0)
        # only c_valid
        self.assertEqual(main_dish_component_line.lqty, 1)

    def test_step_3__clashing_ingredients_restrictions_table(self):
        _ = self._refresh_orders()
        response = self._today_meal()
        self.assertRedirects(
            response,
            reverse("delivery:meal")
            + f"?delivery_date={datetime.date.today().isoformat()}",
        )
        response = self.client.get(
            reverse("delivery:kitchen_count"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        meal_lines = response.context["meal_lines"]

        # contains c_valid (10)
        ml_valid_clash = next(
            ml
            for ml in meal_lines
            if "chicken_10" in ml.ingr_clash and "wine_10" in ml.ingr_clash
        )
        self.assertEqual(ml_valid_clash.rqty, "0")
        self.assertEqual(ml_valid_clash.lqty, "1")
        ml_valid_restrict = next(ml for ml in meal_lines if "veggie_10" in ml.rest_item)
        self.assertIn("Valid", ml_valid_restrict.client)

        # doesn't contain c_nronly (20)
        self.assertRaises(
            StopIteration,
            lambda: next(
                ml
                for ml in meal_lines
                if "chicken_20" in ml.ingr_clash and "wine_20" in ml.ingr_clash
            ),
        )
        self.assertRaises(
            StopIteration,
            lambda: next(ml for ml in meal_lines if "veggie_20" in ml.rest_item),
        )

        # doesn't contain c_ngonly (30)
        self.assertRaises(
            StopIteration,
            lambda: next(
                ml
                for ml in meal_lines
                if "chicken_30" in ml.ingr_clash and "wine_30" in ml.ingr_clash
            ),
        )
        self.assertRaises(
            StopIteration,
            lambda: next(ml for ml in meal_lines if "veggie_30" in ml.rest_item),
        )

        # doesn't contain c_nrng (40)
        self.assertRaises(
            StopIteration,
            lambda: next(
                ml
                for ml in meal_lines
                if "chicken_40" in ml.ingr_clash and "wine_40" in ml.ingr_clash
            ),
        )
        self.assertRaises(
            StopIteration,
            lambda: next(ml for ml in meal_lines if "veggie_40" in ml.rest_item),
        )

        # TOTAL SPECIALS
        ml_last = meal_lines[-1]
        self.assertEqual(ml_last.ingr_clash, "TOTAL SPECIALS")
        self.assertEqual(ml_last.rqty, "0")
        self.assertEqual(ml_last.lqty, "1")

    def test_step_3__labels(self):
        _ = self._refresh_orders()
        response = self._today_meal()
        self.assertRedirects(
            response,
            reverse("delivery:meal")
            + f"?delivery_date={datetime.date.today().isoformat()}",
        )
        response = self.client.get(
            reverse("delivery:kitchen_count"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.context["has_data"], True)

    def test_step_4__routes_before_organizing(self):
        """
        Should ignore route==None and non geolocalized clients.
        When the delivery history doesn't exist or it's invalid,
        don't let route sheet and route sheets work.
        """
        _ = self._refresh_orders()
        _ = self._today_meal()
        response = self.client.get(
            reverse("delivery:routes"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        route_orders_tuples = response.context["route_details"]
        route_orders_dict = dict(map(lambda t: (t[0], t[1:]), route_orders_tuples))
        self.assertIn(self.route1, route_orders_dict)
        self.assertEqual(route_orders_dict[self.route1][0], 1)
        self.assertIn(self.route2, route_orders_dict)
        self.assertEqual(route_orders_dict[self.route2][0], 0)
        # Assert not configured
        self.assertFalse(response.context["all_configured"])
        self.assertFalse(
            DeliveryHistory.objects.filter(
                route=self.route1, date=tz.datetime.today()
            ).exists()
        )
        self.assertFalse(
            DeliveryHistory.objects.filter(
                route=self.route2, date=tz.datetime.today()
            ).exists()
        )
        # Assert print doesn't work
        response = self.client.get(
            reverse("delivery:routes"),
            {"download": "yes", "delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 404)
        # Assert route sheets don't exist
        response = self.client.get(
            reverse("delivery:route_sheet", args=[self.route1.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 404)
        response = self.client.get(
            reverse("delivery:route_sheet", args=[self.route2.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 404)

        # Try invalid DeliveryHistory
        DeliveryHistory.objects.create(
            route=self.route1,
            date=tz.datetime.today(),
            # As long as this sequence is not exactly same as the clients',
            # it's always invalid, even if it contains all the clients.
            client_id_sequence=[999999, 888888, self.c_valid.pk],
        )
        # Assert print doesn't work
        response = self.client.get(
            reverse("delivery:routes"),
            {"download": "yes", "delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 404)

    def test_step_4__routes_after_organizing(self):
        """
        Should ignore route==None and not geolocalized
        """
        _ = self._refresh_orders()
        _ = self._today_meal()
        # Work on Route 1
        response = self.client.post(
            reverse("delivery:create_delivery", args=[self.route1.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertRedirects(
            response,
            reverse("delivery:edit_delivery_route", args=[self.route1.pk])
            + f"?delivery_date={datetime.date.today().isoformat()}",
        )
        self.assertTrue(
            DeliveryHistory.objects.filter(
                route=self.route1, date=tz.datetime.today()
            ).exists()
        )
        response = self.client.get(
            reverse("delivery:edit_delivery_route", args=[self.route1.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 200)
        response = self.client.post(
            reverse("delivery:edit_delivery_route", args=[self.route1.pk]),
            {
                "delivery_date": datetime.date.today().isoformat(),
                "vehicle": "walking",
                "comments": "My bicycle is broken",
                "client_id_sequence": json.dumps([self.c_valid.pk]),
            },
        )
        self.assertRedirects(
            response,
            reverse("delivery:routes")
            + f"?delivery_date={datetime.date.today().isoformat()}",
        )

        # Assert configured because Route 2 is empty.
        response = self.client.get(
            reverse("delivery:routes"),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertTrue(response.context["all_configured"])

        # Assert Route 2 doesn't allow the creation of delivery
        response = self.client.post(
            reverse("delivery:create_delivery", args=[self.route2.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 404)
        self.assertFalse(
            DeliveryHistory.objects.filter(
                route=self.route2, date=tz.datetime.today()
            ).exists()
        )
        response = self.client.get(
            reverse("delivery:edit_delivery_route", args=[self.route2.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 404)
        response = self.client.post(
            reverse("delivery:edit_delivery_route", args=[self.route2.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response.status_code, 404)

        # Assert print works
        response = self.client.get(
            reverse("delivery:routes"),
            {"delivery_date": datetime.date.today().isoformat(), "download": "yes"},
        )
        self.assertEqual(response.status_code, 200)
        # Check print result
        routes_dict = json.loads(response["routes_dict"])
        # only route1 has one client
        route1 = routes_dict[str(self.route1.id)]
        self.assertEqual(len(route1["detail_lines"]), 1)
        # route2 doesn't exist.
        self.assertNotIn(str(self.route2.id), routes_dict)

        # check output page : use data in header because cannot read PDF
        route_name_list = [item["route"] for item in routes_dict.values()]
        self.assertIn(self.route1.name, route_name_list)
        self.assertNotIn(self.route2.name, route_name_list)

        # Assert route sheets exist and check route sheets
        # Route 1
        response1 = self.client.get(
            reverse("delivery:route_sheet", args=[self.route1.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response1.status_code, 200)
        # 1 dessert 1 L main_dish
        summary_line1 = response1.context["summary_lines"]
        main_dish_line1 = next(
            ln for ln in summary_line1 if ln.component_group == "main_dish"
        )
        self.assertEqual(main_dish_line1.rqty, 0)
        self.assertEqual(main_dish_line1.lqty, 1)
        dessert_line1 = next(
            ln for ln in summary_line1 if ln.component_group == "dessert"
        )
        self.assertEqual(dessert_line1.rqty, 1)
        # 1 client
        detail_line1 = response1.context["detail_lines"]
        self.assertEqual(len(detail_line1), 1)
        self.assertEqual(detail_line1[0].firstname, "Valid")
        self.assertEqual(detail_line1[0].lastname, "Valid")

        # Route 2 - not found
        response2 = self.client.get(
            reverse("delivery:route_sheet", args=[self.route2.pk]),
            {"delivery_date": datetime.date.today().isoformat()},
        )
        self.assertEqual(response2.status_code, 404)


class RouteSheetReportTestCase(SousChefTestMixin, TestCase):
    # PDF route sheets report generation
    fixtures = ["sample_data"]

    def setUp(self):
        self.force_login()

    def test_route_sheet_report(self):
        # generate orders today
        Order.objects.auto_create_orders(datetime.date.today(), Client.active.all())

        # list all the routes
        response = self.client.get(
            reverse("delivery:routes"), {"delivery_date": datetime.date.today()}
        )
        self.assertEqual(response.status_code, 200)
        # get details of each route : {route:(order_count, ...), ...}
        route_orders_dict = dict(
            map(lambda t: (t[0], t[1:]), response.context["route_details"])
        )

        # create history and organize clients for the non empty routes
        for route, num in route_orders_dict.items():
            if num[0] > 0:
                # route has clients with orders : create delivery history
                response = self.client.post(
                    reverse("delivery:create_delivery", args=[route.pk]),
                    {"delivery_date": datetime.date.today().isoformat()},
                )
                self.assertRedirects(
                    response,
                    reverse("delivery:edit_delivery_route", args=[route.pk])
                    + f"?delivery_date={datetime.date.today().isoformat()}",
                )

                # get the clients in the route
                response = self.client.get(
                    reverse("delivery:edit_delivery_route", args=[route.pk]),
                    {"delivery_date": datetime.date.today()},
                )
                self.assertEqual(response.status_code, 200)
                client_pks = [
                    item.pk for item in response.context["clients_on_delivery_history"]
                ]

                # organize the clients in the route
                response = self.client.post(
                    reverse("delivery:edit_delivery_route", args=[route.pk]),
                    {
                        "vehicle": "walking",
                        "comments": "Nice day !",
                        "client_id_sequence": json.dumps(client_pks),
                        "delivery_date": datetime.date.today().isoformat(),
                    },
                )
                self.assertRedirects(
                    response,
                    reverse("delivery:routes")
                    + f"?delivery_date={datetime.date.today().isoformat()}",
                )

        # generate the PDF route sheets of all the routes
        response = self.client.get(
            reverse("delivery:routes"),
            {
                "download": "yes",
                "delivery_date": datetime.date.today().isoformat(),
            },
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue("ReportLab" in repr(response.content))



import rules
from rules.predicates import (
    is_authenticated,
    is_staff,
    is_superuser,
)

# Note: only two global "permissions" are considered in this project for now.
# "read" and "edit". Should fine-grained permissions be needed, these basic
# permissions should be removed and replaced by permissions associated with
# each app.
rules.add_perm("sous_chef.edit", is_authenticated & is_superuser)
rules.add_perm("sous_chef.read", is_authenticated & is_staff)



# Generated by Django 1.9.5 on 2016-06-05 19:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Component",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
                (
                    "component_group",
                    models.CharField(
                        choices=[
                            ("main dish", "main dish"),
                            ("vegetable", "Vegetable"),
                            ("side dish", "side dish"),
                            ("seasonal", "Seasonal"),
                        ],
                        max_length=100,
                        verbose_name="component group",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "components",
            },
        ),
        migrations.CreateModel(
            name="Component_ingredient",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "component",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="meal.Component",
                        verbose_name="component",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Incompatibility",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Ingredient",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
            ],
            options={
                "verbose_name_plural": "meals",
            },
        ),
        migrations.CreateModel(
            name="Menu",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(verbose_name="date")),
            ],
            options={
                "verbose_name_plural": "menus",
            },
        ),
        migrations.CreateModel(
            name="Menu_component",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "component",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="meal.Component",
                        verbose_name="component",
                    ),
                ),
                (
                    "menu",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="meal.Menu",
                        verbose_name="menu",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Restricted_item",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
                (
                    "restricted_item_group",
                    models.CharField(
                        choices=[
                            ("meat", "Meat"),
                            ("vegetables", "Vegetables"),
                            ("seafood", "Seafood"),
                            ("seeds and nuts", "Seeds and nuts"),
                            ("other", "Other"),
                        ],
                        max_length=100,
                        verbose_name="restricted item group",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "restricted items",
            },
        ),
        migrations.AddField(
            model_name="incompatibility",
            name="ingredient",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="meal.Ingredient",
                verbose_name="ingredient",
            ),
        ),
        migrations.AddField(
            model_name="incompatibility",
            name="restricted_item",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="meal.Restricted_item",
                verbose_name="restricted item",
            ),
        ),
        migrations.AddField(
            model_name="component_ingredient",
            name="ingredient",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="meal.Ingredient",
                verbose_name="ingredient",
            ),
        ),
    ]



# Generated by Django 1.10.6 on 2017-03-28 18:28

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0030_route_deliveryhistory"),
    ]

    operations = [
        migrations.AlterField(
            model_name="client_option",
            name="client",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Client",
                verbose_name="client",
            ),
        ),
        migrations.AlterField(
            model_name="client_option",
            name="option",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Option",
                verbose_name="option",
            ),
        ),
    ]



from django.core.management.base import BaseCommand

from souschef.member.factories import NoteFactory


class Command(BaseCommand):
    help = "Creates notes related to members."

    def handle(self, *args, **options):
        notes = NoteFactory.create_batch(20)
        self.stdout.write(self.style.SUCCESS('Successfully created notes "%s"' % notes))



import csv

from django.core.management.base import BaseCommand

from souschef.member.models import (
    EMAIL,
    HOME,
    Address,
    Contact,
    Member,
)


class Command(BaseCommand):
    help = "Data: import clients from given csv file."

    ROW_MID = 0
    ROW_ADDRESS1 = 3
    ROW_APARTMENT = 4
    ROW_CITY = 5
    ROW_POSTAL_CODE = 6
    ROW_PHONE = 7
    ROW_EMAIL = 8

    def add_arguments(self, parser):
        parser.add_argument(
            "--file",
            default=False,
            help="Import mock data instead of actual data",
        )

    def handle(self, *args, **options):
        file = "mock_addresses.csv" if options["file"] else "clients_address.csv"

        with open(file) as f:
            reader = csv.reader(f, delimiter=";")
            for row in reader:
                try:
                    member = Member.objects.get(mid=row[self.ROW_MID])
                    # if member.address is not None:
                    #    member.address.delete()
                    address = Address.objects.create(
                        street=row[self.ROW_ADDRESS1],
                        apartment=row[self.ROW_APARTMENT],
                        city=row[self.ROW_CITY],
                        postal_code=row[self.ROW_POSTAL_CODE],
                    )

                    member.address = address
                    member.save()

                    contacts = Contact.objects.filter(member=member)
                    contacts.delete()

                    if row[self.ROW_PHONE] != "":
                        Contact.objects.create(
                            type=HOME, value=row[self.ROW_PHONE], member=member
                        )
                    if row[self.ROW_EMAIL] != "":
                        Contact.objects.create(
                            type=EMAIL, value=row[self.ROW_EMAIL], member=member
                        )

                except Member.DoesNotExist:
                    self.stdout.write(self.style.WARNING("Non existing member"))



# usage : python dataexec.py [santropolFeast.settingsSPECIAL]
import os
import sys


def run():
    settings = "souschef.sous_chef.settings"
    if len(sys.argv) > 1:
        settings = sys.argv[1]
    os.environ["DJANGO_SETTINGS_MODULE"] = settings
    import django

    django.setup()
    from dataload import insert_all

    insert_all()


run()



# Generated by Django 1.10 on 2016-11-25 20:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0020_auto_20161122_0458"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="postal_code",
            field=models.CharField(max_length=7, verbose_name="postal code"),
        ),
    ]



# Generated by Django 1.9.5 on 2016-08-03 11:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0004_fix004d"),
    ]

    operations = [
        migrations.AlterField(
            model_name="component",
            name="component_group",
            field=models.CharField(
                choices=[
                    ("main_dish", "Main Dish"),
                    ("dessert", "Dessert"),
                    ("diabetic", "Diabetic Dessert"),
                    ("fruit_salad", "Fruit Salad"),
                    ("green_salad", "Green Salad"),
                    ("pudding", "Pudding"),
                    ("compote", "Compote"),
                ],
                max_length=100,
                verbose_name="component group",
            ),
        ),
        migrations.AlterField(
            model_name="component",
            name="name",
            field=models.CharField(max_length=50, unique=True, verbose_name="name"),
        ),
        migrations.AlterField(
            model_name="ingredient",
            name="ingredient_group",
            field=models.CharField(
                choices=[
                    ("meat", "Meat"),
                    ("dairy", "Dairy"),
                    ("fish", "Fish"),
                    ("seafood", "Seafood"),
                    ("veggies_and_fruits", "Veggies and fruits"),
                    ("legumineuse", "Legumineuse"),
                    ("grains", "Grains"),
                    ("fresh_herbs", "Fresh herbs"),
                    ("spices", "Spices"),
                    ("dry_and_canned_goods", "Dry and canned goods"),
                    ("oils_and_sauces", "Oils and sauces"),
                ],
                max_length=100,
                verbose_name="ingredient group",
            ),
        ),
        migrations.AlterField(
            model_name="ingredient",
            name="name",
            field=models.CharField(max_length=50, unique=True, verbose_name="name"),
        ),
    ]



# Generated by Django 1.11.29 on 2021-03-05 15:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0036_member_addresses"),
    ]

    operations = [
        migrations.AlterField(
            model_name="member",
            name="address",
            field=models.OneToOneField(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="member.Address",
                verbose_name="address",
            ),
        ),
    ]



# Generated by Django 1.10 on 2016-10-12 19:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0009_auto_20161012_1919"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="creation_date",
            field=models.DateField(auto_now_add=True, verbose_name="creation date"),
        ),
    ]



# Generated by Django 1.9.7 on 2016-07-26 17:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0006_client_meal_default_week"),
    ]

    operations = [
        migrations.AddField(
            model_name="address",
            name="distance",
            field=models.DecimalField(
                blank=True, decimal_places=6, max_digits=9, null=True
            ),
        ),
        migrations.AddField(
            model_name="address",
            name="lat",
            field=models.DecimalField(
                blank=True, decimal_places=6, max_digits=9, null=True
            ),
        ),
        migrations.AddField(
            model_name="address",
            name="lon",
            field=models.DecimalField(
                blank=True, decimal_places=6, max_digits=9, null=True
            ),
        ),
    ]



# Generated by Django 1.10 on 2016-09-12 15:09

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0007_remove_order_item_component"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="client",
            field=models.ForeignKey(
                default="",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="client_order",
                to="member.Client",
                verbose_name="client",
            ),
        ),
    ]



# Generated by Django 1.11.3 on 2017-08-01 20:07

import annoying.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0032_relationship"),
    ]

    operations = [
        migrations.AlterField(
            model_name="relationship",
            name="extra_fields",
            field=annoying.fields.JSONField(blank=True, default={}),
        ),
        migrations.AlterField(
            model_name="relationship",
            name="remark",
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name="relationship",
            name="type",
            field=annoying.fields.JSONField(blank=True, default=[]),
        ),
    ]



# Generated by Django 1.10.5 on 2017-03-13 18:42

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0026_change_to_emergency_contacts"),
    ]

    operations = [
        migrations.AlterField(
            model_name="client",
            name="billing_member",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="member.Member",
                verbose_name="billing member",
            ),
        ),
        migrations.AlterField(
            model_name="member",
            name="address",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="member.Address",
                verbose_name="address",
            ),
        ),
        migrations.AlterField(
            model_name="route",
            name="vehicle",
            field=models.CharField(
                choices=[
                    ("cycling", "Cycling"),
                    ("walking", "Walking"),
                    ("driving", "Driving"),
                ],
                default="cycling",
                max_length=20,
                verbose_name="vehicle",
            ),
        ),
    ]



# Generated by Django 1.10 on 2016-10-20 20:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0016_auto_20160912_1509"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="route",
            options={"ordering": ["name"], "verbose_name_plural": "Routes"},
        ),
        migrations.AlterField(
            model_name="client_option",
            name="value",
            field=models.CharField(max_length=255, null=True, verbose_name="value"),
        ),
    ]



# Generated by Django 1.10 on 2016-12-27 18:19

import collections

from django.db import migrations, models

work_information = collections.defaultdict(list)


def copy_work_information(apps, schema_editor):
    Referencing = apps.get_model("member", "Referencing")
    for ref in Referencing.objects.all():
        if ref.referent and ref.work_information:
            member_id = ref.referent_id
            work = ref.work_information
            work_information[member_id].append(work)
    # check the data
    for member_id, works in work_information.items():
        if len(works) > 1:
            # warning
            print(
                f"Warning: Member #{member_id} has multiple work information registered"
                "in Referencing table. They are:"
            )
            for work in works:
                print(f" - {work}")
            print("")
            answer = input("Please enter the final work information: ")
            works[0] = answer


def paste_work_information(apps, schema_editor):
    Member = apps.get_model("member", "Member")
    for member_id, works in work_information.items():
        member = Member.objects.get(id=member_id)
        member.work_information = works[0]
        member.save(update_fields=["work_information"])


# Reverse codes for RunPython commands
undo_work_information = {}


def undo_paste_work_information(apps, schema_editor):
    Member = apps.get_model("member", "Member")
    for member in Member.objects.all():
        if member.work_information:
            undo_work_information[member.id] = member.work_information


def undo_copy_work_information(apps, schema_editor):
    Referencing = apps.get_model("member", "Referencing")
    for member_id, work in undo_work_information.items():
        refs = Referencing.objects.filter(referent=member_id)
        for ref in refs:
            ref.work_information = work
            ref.save(update_fields=["work_information"])


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0023_auto_20161220_1401"),
    ]

    operations = [
        migrations.RunPython(
            copy_work_information, reverse_code=undo_copy_work_information
        ),
        migrations.RemoveField(
            model_name="referencing",
            name="work_information",
        ),
        migrations.AddField(
            model_name="member",
            name="work_information",
            field=models.TextField(
                blank=True, null=True, verbose_name="Work information"
            ),
        ),
        migrations.RunPython(
            paste_work_information, reverse_code=undo_paste_work_information
        ),
    ]



# Generated by Django 1.10 on 2016-08-20 23:22

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0012_clientscheduledstatus"),
    ]

    operations = [
        migrations.AlterField(
            model_name="clientscheduledstatus",
            name="change_date",
            field=models.DateField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="clientscheduledstatus",
            name="change_state",
            field=models.CharField(
                choices=[("START", "Start"), ("END", "End")],
                default="START",
                max_length=5,
            ),
        ),
    ]



# Generated by Django 1.9.7 on 2016-08-03 20:52

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0009_auto_20160728_1443"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="note",
            name="author",
        ),
        migrations.RemoveField(
            model_name="note",
            name="member",
        ),
        migrations.DeleteModel(
            name="Note",
        ),
    ]



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from souschef.note.views import (
    NoteAdd,
    NoteBatchToggle,
    NoteDeleteView,
    NoteEditView,
    NoteList,
    mark_as_read,
    mark_as_unread,
)

app_name = "note"

urlpatterns = [
    url(_(r"^$"), NoteList.as_view(), name="note_list"),
    url(_(r"^read/(?P<id>\d+)/$"), mark_as_read, name="read"),
    url(_(r"^unread/(?P<id>\d+)/$"), mark_as_unread, name="unread"),
    url(_(r"^add/$"), NoteAdd.as_view(), name="note_add"),
    url(_(r"^batch_toggle/$"), NoteBatchToggle.as_view(), name="batch_toggle"),
    url(_(r"^edit/(?P<pk>\d+)/$"), NoteEditView.as_view(), name="edit"),
    url(
        _(r"^delete/(?P<pk>\d+)/$"),
        NoteDeleteView.as_view(),
        name="note_delete",
    ),
]



from django.http import JsonResponse


class FormInvalidAjaxableResponseMixin:
    """
    Only ajaxable when the form is invalid: returning errors.
    """

    def form_invalid(self, form):
        response = super().form_invalid(form)
        if self.request.is_ajax():
            return JsonResponse(form.errors, status=400)
        else:
            return response


class FormValidAjaxableResponseMixin:
    """
    Only ajaxable when the form is valid: returning pk.
    """

    def form_valid(self, form):
        # We make sure to call the parent's form_valid() method because
        # it might do some processing (in the case of CreateView, it will
        # call form.save() for example).
        response = super().form_valid(form)
        if self.request.is_ajax():
            data = {
                "pk": self.object.pk,
            }
            return JsonResponse(data)
        else:
            return response


class AjaxableResponseMixin(
    FormInvalidAjaxableResponseMixin, FormValidAjaxableResponseMixin
):
    """
    Mixin to add AJAX support to a form.
    Must be used with an object-based FormView (e.g. CreateView)
    https://docs.djangoproject.com/ja/1.9/topics/class-based-views/generic-editing/#ajax-example
    """

    pass



from django import forms
from django.db.models.functions import Lower
from django.utils.translation import ugettext_lazy as _

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES_MAIN_DISH,
    Component,
    Ingredient,
)


class DishIngredientsForm(forms.Form):
    maindish = forms.ModelChoiceField(
        label=_("Today's main dish:"),
        queryset=Component.objects.order_by(Lower("name")).filter(
            component_group=COMPONENT_GROUP_CHOICES_MAIN_DISH
        ),
        widget=forms.Select(attrs={"class": "ui dropdown maindish"}),
    )

    ingredients = forms.ModelMultipleChoiceField(
        label=_("Select main dish ingredients:"),
        queryset=Ingredient.objects.order_by(Lower("name")).all(),
        widget=forms.SelectMultiple(
            attrs={"class": "ui fluid search dropdown mainingredients"}
        ),
        required=False,
    )

    sides_ingredients = forms.ModelMultipleChoiceField(
        label=_("Select sides ingredients:"),
        queryset=Ingredient.objects.order_by(Lower("name")).all(),
        widget=forms.SelectMultiple(
            attrs={"class": "ui fluid search dropdown sidesingredients"}
        ),
        required=False,
    )

    def clean_sides_ingredients(self):
        data = self.cleaned_data["sides_ingredients"]
        if not data:
            raise forms.ValidationError(_("Please choose some Sides ingredients"))
        return data



import random

import factory

from souschef.meal.models import COMPONENT_GROUP_CHOICES
from souschef.member.models import (
    DAYS_OF_WEEK,
    DELIVERY_TYPE,
    GENDER_CHOICES,
    PAYMENT_TYPE,
    RATE_TYPE,
    Address,
    Client,
    ClientScheduledStatus,
    Contact,
    DeliveryHistory,
    Member,
    Relationship,
    Route,
)


class AddressFactory(factory.DjangoModelFactory):
    class Meta:
        model = Address

    street = factory.Faker("street_address")
    apartment = factory.Faker("random_number")
    city = "Montreal"
    postal_code = factory.Faker("postalcode", locale="en_CA")
    latitude = factory.LazyAttribute(lambda x: random.choice(["75.0", "75.1", "75.2"]))
    longitude = factory.LazyAttribute(lambda x: random.choice(["40.2", "40.1", "40.0"]))


class MemberFactory(factory.DjangoModelFactory):
    class Meta:
        model = Member

    firstname = factory.Faker("first_name")
    lastname = factory.Faker("last_name")

    address = factory.SubFactory(AddressFactory)
    work_information = factory.Faker("company")
    contact = factory.RelatedFactory("member.factories.ContactFactory", "member")


class RouteFactory(factory.DjangoModelFactory):
    class Meta:
        model = Route

    name = factory.Faker("name")


class DeliveryHistoryFactory(factory.DjangoModelFactory):
    class Meta:
        model = DeliveryHistory


def generate_json():
    json = {}
    for day, _translation in DAYS_OF_WEEK:
        json[f"size_{day}"] = random.choice(["L", "R"])
        for meal, _Meal in COMPONENT_GROUP_CHOICES:
            json[f"{meal}_{day}_quantity"] = random.choice([0, 1])
    return json


class ClientFactory(factory.DjangoModelFactory):
    class Meta:
        model = Client

    member = factory.SubFactory(MemberFactory)
    billing_member = member
    billing_payment_type = factory.LazyAttribute(
        lambda x: random.choice(PAYMENT_TYPE)[0]
    )
    rate_type = factory.LazyAttribute(lambda x: random.choice(RATE_TYPE)[0])
    member = member
    status = factory.LazyAttribute(lambda x: random.choice(Client.CLIENT_STATUS)[0])
    language = factory.LazyAttribute(lambda x: random.choice(Client.LANGUAGES)[0])
    alert = factory.Faker("sentence")
    delivery_type = factory.LazyAttribute(lambda x: random.choice(DELIVERY_TYPE)[0])
    gender = factory.LazyAttribute(lambda x: random.choice(GENDER_CHOICES)[0])
    birthdate = factory.Faker("date")
    route = factory.LazyAttribute(lambda x: random.choice(Route.objects.all()))

    delivery_note = factory.Faker("sentence")

    meal_default_week = factory.LazyAttribute(lambda x: generate_json())


def random_combination(iterable, r):
    """
    Random selection from itertools.combinations(iterable, r)
    From Python docs (itertools)
    """
    pool = tuple(iterable)
    n = len(pool)
    indices = sorted(random.sample(range(n), r))
    return tuple(pool[i] for i in indices)


class RelationshipFactory(factory.DjangoModelFactory):
    class Meta:
        model = Relationship

    client = factory.SubFactory(ClientFactory)
    member = factory.SubFactory(MemberFactory)
    nature = factory.LazyAttribute(
        lambda x: random.choice(["friends", "family", "coworkers"])
    )
    type = factory.LazyAttribute(
        lambda x: list(
            random_combination(
                map(lambda tup: tup[0], Relationship.TYPE_CHOICES),
                random.randint(0, len(Relationship.TYPE_CHOICES)),
            )
        )
    )
    extra_fields = factory.LazyAttribute(lambda x: {})
    remark = factory.Faker("sentence")


class ContactFactory(factory.DjangoModelFactory):
    class Meta:
        model = Contact

    type = "Home phone"
    value = factory.Sequence(lambda n: "514-555-%04d" % n)
    member = factory.SubFactory(MemberFactory)


class ClientScheduledStatusFactory(factory.DjangoModelFactory):
    class Meta:
        model = ClientScheduledStatus

    client = factory.SubFactory(ClientFactory)
    status_from = factory.LazyAttribute(
        lambda x: random.choice(Client.CLIENT_STATUS)[0]
    )
    status_to = factory.LazyAttribute(lambda x: random.choice(Client.CLIENT_STATUS)[0])
    reason = factory.Faker("sentence")
    change_date = factory.Faker("date")
    change_state = factory.LazyAttribute(
        lambda x: random.choice(ClientScheduledStatus.CHANGE_STATUS)[0]
    )
    operation_status = ClientScheduledStatus.TOBEPROCESSED



# Generated by Django 1.9.5 on 2016-08-31 17:09

import annoying.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0014_auto_20160826_0007"),
    ]

    operations = [
        migrations.AddField(
            model_name="route",
            name="client_id_sequence",
            field=annoying.fields.JSONField(blank=True, null=True),
        ),
    ]



from django.conf.urls import url
from django.utils.translation import string_concat
from django.utils.translation import ugettext_lazy as _

from souschef.member.forms import (
    ClientAddressInformation,
    ClientBasicInformation,
    ClientPaymentInformation,
    ClientRestrictionsInformation,
)
from souschef.member.formsets import CreateRelationshipFormset
from souschef.member.views import (
    ClientAllergiesView,
    ClientInfoView,
    ClientList,
    ClientOrderList,
    ClientPaymentView,
    ClientStatusScheduler,
    ClientStatusSchedulerDeleteView,
    ClientStatusSchedulerReschedule,
    ClientStatusView,
    ClientUpdateAddressInformation,
    ClientUpdateBasicInformation,
    ClientUpdateDietaryRestriction,
    ClientUpdatePaymentInformation,
    ClientUpdateRelationshipsInformation,
    ClientWizard,
    DeleteClientOption,
    DeleteComponentToAvoid,
    DeleteIngredientToAvoid,
    DeleteRestriction,
    DeliveryHistoryDetailView,
    RouteDetailView,
    RouteEditView,
    RouteListView,
    SearchMembers,
    geolocateAddress,
    get_minimised_euclidean_distances_route_sequence,
)
from souschef.note.views import (
    ClientNoteList,
    ClientNoteListAdd,
)

app_name = "member"

member_forms = (
    {
        "name": "basic_information",
        "step_url": _("basic_information"),
        "create_form": ClientBasicInformation,
        "update_form": ClientUpdateBasicInformation,
    },
    {
        "name": "address_information",
        "step_url": _("address_information"),
        "create_form": ClientAddressInformation,
        "update_form": ClientUpdateAddressInformation,
    },
    {
        "name": "relationships",
        "step_url": _("relationships"),
        "create_form": CreateRelationshipFormset,
        "update_form": ClientUpdateRelationshipsInformation,
    },
    {
        "name": "payment_information",
        "step_url": _("payment_information"),
        "create_form": ClientPaymentInformation,
        "update_form": ClientUpdatePaymentInformation,
    },
    {
        "name": "dietary_restriction",
        "step_url": _("dietary_restriction"),
        "create_form": ClientRestrictionsInformation,
        "update_form": ClientUpdateDietaryRestriction,
    },
)

member_wizard = ClientWizard.as_view(
    list(map(lambda d: (d["name"], d["create_form"]), member_forms)),
    i18n_url_names=list(map(lambda d: (d["name"], d["step_url"]), member_forms)),
    url_name="member:member_step",
)


urlpatterns = [
    url(_(r"^create/$"), member_wizard, name="member_step"),
    url(_(r"^create/(?P<step>.+)/$"), member_wizard, name="member_step"),
    url(_(r"^list/$"), ClientList.as_view(), name="list"),
    url(_(r"^search/$"), SearchMembers.as_view(), name="search"),
    url(_(r"^view/(?P<pk>\d+)/orders$"), ClientOrderList.as_view(), name="list_orders"),
    url(
        _(r"^view/(?P<pk>\d+)/information$"),
        ClientInfoView.as_view(),
        name="client_information",
    ),
    url(
        _(r"^view/(?P<pk>\d+)/billing$"),
        ClientPaymentView.as_view(),
        name="client_payment",
    ),
    url(
        _(r"^view/(?P<pk>\d+)/preferences$"),
        ClientAllergiesView.as_view(),
        name="client_allergies",
    ),
    url(_(r"^view/(?P<pk>\d+)/notes$"), ClientNoteList.as_view(), name="client_notes"),
    url(
        _(r"^view/(?P<pk>\d+)/notes/add$"),
        ClientNoteListAdd.as_view(),
        name="client_notes_add",
    ),
    url(_(r"^geolocateAddress/$"), geolocateAddress, name="geolocateAddress"),
    url(
        _(r"^view/(?P<pk>\d+)/status$"),
        ClientStatusView.as_view(),
        name="client_status",
    ),
    url(
        _(r"^client/(?P<pk>\d+)/status/scheduler$"),
        ClientStatusScheduler.as_view(),
        name="clientStatusScheduler",
    ),
    url(
        _(
            r"^client/(?P<pk>\d+)/status/scheduler/reschedule/"
            r"(?P<scheduled_status_1_pk>\d+)"
            r",(?P<scheduled_status_2_pk>\d+)?"
            r"/$"
        ),
        ClientStatusSchedulerReschedule.as_view(),
        name="clientStatusSchedulerReschedule",
    ),
    url(
        r"^status/(?P<pk>\d+)/delete$",
        ClientStatusSchedulerDeleteView.as_view(),
        name="delete_status",
    ),
    url(
        _(r"^restriction/(?P<pk>\d+)/delete/$"),
        DeleteRestriction.as_view(),
        name="restriction_delete",
    ),
    url(
        _(r"^client_option/(?P<pk>\d+)/delete/$"),
        DeleteClientOption.as_view(),
        name="client_option_delete",
    ),
    url(
        _(r"^ingredient_to_avoid/(?P<pk>\d+)/delete/$"),
        DeleteIngredientToAvoid.as_view(),
        name="ingredient_to_avoid_delete",
    ),
    url(
        _(r"^component_to_avoid/(?P<pk>\d+)/delete/$"),
        DeleteComponentToAvoid.as_view(),
        name="component_to_avoid_delete",
    ),
    url(_(r"^routes/$"), RouteListView.as_view(), name="route_list"),
    url(_(r"^route/(?P<pk>\d+)/$"), RouteDetailView.as_view(), name="route_detail"),
    url(_(r"^route/(?P<pk>\d+)/edit/$"), RouteEditView.as_view(), name="route_edit"),
    url(
        _(r"^route/(?P<pk>\d+)/optimised_sequence/$"),
        get_minimised_euclidean_distances_route_sequence,
        name="route_get_optimised_sequence",
    ),
    url(
        _(r"^route/(?P<route_pk>\d+)/(?P<date>\d{4}-\d{2}-\d{2})/$"),
        DeliveryHistoryDetailView.as_view(),
        name="delivery_history_detail",
    ),
]

# Handle client update forms URL
for d in member_forms:
    urlpatterns.append(
        url(
            string_concat(_(r"^(?P<pk>\d+)/update/"), d["step_url"], "/$"),
            d["update_form"].as_view(),
            name="member_update_" + d["name"],
        )
    )



# Generated by Django 1.11.29 on 2021-01-15 16:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0034_auto_20170816_0850"),
    ]

    operations = [
        migrations.AlterField(
            model_name="client",
            name="billing_payment_type",
            field=models.CharField(
                blank=True,
                choices=[
                    (" ", "----"),
                    ("3rd", "3rd Party"),
                    ("credit", "Credit card"),
                    ("eft", "EFT"),
                    ("cash", "Cash"),
                    ("etransfert", "e-Transfer"),
                    ("cheque", "Cheque"),
                ],
                max_length=10,
                null=True,
                verbose_name="Payment Type",
            ),
        ),
    ]



# Generated by Django 1.9.7 on 2016-07-27 22:51

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0007_auto_20160726_1703"),
    ]

    operations = [
        migrations.RenameField(
            model_name="address",
            old_name="lat",
            new_name="latitude",
        ),
        migrations.RenameField(
            model_name="address",
            old_name="lon",
            new_name="longitude",
        ),
    ]



from django.db import models
from django.utils.translation import ugettext_lazy as _

# Create your models here.


class Delivery(models.Model):  # noqa: DJ008
    class Meta:
        verbose_name_plural = _("deliveries")

    pass



# Generated by Django 1.11.29 on 2021-07-09 18:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0011_auto_20170419_1109"),
    ]

    operations = [
        migrations.AddField(
            model_name="note",
            name="is_deleted",
            field=models.BooleanField(default=False, verbose_name="Is deleted"),
        ),
    ]



import itertools


class Node:
    def __init__(self, id, latitude, longitude):
        self.id = id
        self.latitude = latitude
        self.longitude = longitude

    def __str__(self):
        return f"Node({self.id}, {self.latitude}, {self.longitude})"

    def __repr__(self):
        return str(self)


class Solution:
    def __init__(self, tour, value):
        self.tour = tour
        self.value = value


def pairwise(iterable):
    """s -> (s0,s1), (s1,s2), (s2, s3), ...
    Source:
    https://docs.python.org/3.5/library/itertools.html#itertools-recipes
    """
    a, b = itertools.tee(iterable)
    next(b, None)
    return zip(a, b)


def reverse_subtour(tour, start, end):
    """Returns a new list with the subtour reversed"""
    return tour[:start] + list(reversed(tour[start : end + 1])) + tour[end + 1 :]


def solve(tour):
    """Solves the Traveling Salesman Problem (TSP) with a heuristic.

    Args:
        tour: List of nodes (Node) representing a tour. Even if a tour
            starts and ends at the same node, the last node in the
            list must be the last destination visited before returning
            to the starting point.

    Returns:
        A tour with a distance less or equal to the distance of the
        initial tour.

    """

    # This function implements a local search heuristic with a 2-opt
    # neighborhood to solve an Euclidean TSP.

    best_solution = Solution(list(tour), tour_squared_distance(tour))

    improved = True
    while improved:
        improved = False
        best_candidate = Solution(best_solution.tour, best_solution.value)

        for candidate_tour in two_opt_neighbors(best_solution.tour):
            candidate = Solution(candidate_tour, tour_squared_distance(candidate_tour))

            # We use the squared distance since the square root
            # function is monotone increasing, so comparing squared
            # distance is equivalent to comparing distances.
            if candidate.value < best_candidate.value:
                best_candidate = candidate

        if best_candidate.value < best_solution.value:
            improved = True
            best_solution = best_candidate

    return best_solution.tour


def squared_distance(a, b):
    """Squared euclidean distance"""

    return (a.latitude - b.latitude) ** 2 + (a.longitude - b.longitude) ** 2


def tour_squared_distance(tour):
    # "+ tour[0]" => add the distance for returning to the starting
    # point
    distance = 0
    for a, b in pairwise(tour + [tour[0]]):
        distance += squared_distance(a, b)
    return distance


def two_opt_neighbors(tour):
    """Generates 2-opt neighbors for a tour.

    https://en.wikipedia.org/wiki/2-opt
    """
    for start in range(1, len(tour) - 1):
        for end in range(start + 1, len(tour)):
            yield reverse_subtour(tour, start, end)



from datetime import date

from django.core.management import call_command
from django.test import TestCase

from souschef.member.models import (
    Address,
    Client,
    Member,
    Relationship,
)
from souschef.order.models import Order


class ImportMemberTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ["routes.json"]

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """
        call_command("importclients", file="mock_clients.csv")

    def test_import_member_info(self):
        self.assertEqual(10, Client.objects.all().count())
        self.assertEqual(10, Member.objects.all().count())
        dorothy = Client.objects.get(member__mid=94)
        created = dorothy.member.created_at
        self.assertEqual(dorothy.member.firstname, "Dorothy")
        self.assertEqual(dorothy.member.lastname, "Davis")
        self.assertEqual(dorothy.status, Client.ACTIVE)
        self.assertEqual(dorothy.birthdate, date(1938, 10, 10))
        self.assertEqual(
            date(created.year, created.month, created.day), date(2007, 9, 26)
        )
        self.assertEqual(dorothy.gender, "F")
        self.assertEqual(dorothy.language, "EN")
        self.assertEqual(dorothy.delivery_type, Client.ONGOING_DELIVERY)
        self.assertEqual(dorothy.route.name, "McGill")
        self.assertEqual(dorothy.delivery_note, "Code entree: 17")
        self.assertEqual(dorothy.alert, "communicate with her sister")

    def test_import_member_status(self):
        self.assertEqual(Client.active.all().count(), 2)
        self.assertEqual(Client.ongoing.all().count(), 2)
        self.assertEqual(Client.pending.all().count(), 1)
        self.assertEqual(Client.contact.all().count(), 6)

    def test_import_member_routes(self):
        self.assertEqual(Client.objects.filter(route__name="McGill").count(), 2)
        self.assertEqual(Client.objects.filter(route__name="Westmount").count(), 2)
        self.assertEqual(
            Client.objects.filter(route__name="Notre Dame de Grace").count(), 3
        )
        self.assertEqual(
            Client.objects.filter(route__name="Côte-des-Neiges").count(), 2
        )
        self.assertEqual(
            Client.objects.filter(route__name="Centre-Ville / Downtown").count(), 1
        )


class ImportMemberAddressesTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ["routes.json"]

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """
        call_command("importclients", file="mock_clients.csv")
        call_command("importaddresses", file="mock_addresses.csv")

    def test_import_member_addresses(self):
        self.assertEqual(Address.objects.all().count(), 10)
        dorothy = Member.objects.get(mid=94)
        self.assertEqual(dorothy.address.street, "2070 De Maisoneuve Ouest")
        self.assertEqual(dorothy.address.apartment, "Apt.ABC")
        self.assertEqual(dorothy.address.postal_code, "H3G1K9")
        self.assertEqual(dorothy.address.city, "Montreal")
        self.assertEqual(dorothy.home_phone, "514-666-6666")
        self.assertEqual(dorothy.email, "ddavis@example.org")

    def test_ut8_charset(self):
        norma = Member.objects.get(mid=91)
        self.assertEqual(norma.address.city, "Montréal")
        robin = Member.objects.get(mid=99)
        self.assertEqual(robin.address.street, "29874 Côte-des-Neiges")


class ImportMemberRelationshipsTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ["routes.json"]

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """
        call_command("importclients", file="mock_clients.csv")
        call_command("importrelationships", file="mock_relationships.csv")

    def test_import_relationship(self):
        self.assertEqual(10, Client.objects.all().count())
        self.assertEqual(12, Member.objects.all().count())
        dorothy = Client.objects.get(member__mid=94)
        self.assertEqual(str(dorothy.billing_member), "Alice Cardona")
        self.assertEqual(dorothy.billing_member.rid, 2884)
        self.assertIn(
            dorothy.billing_member.pk,
            [Relationship.objects.get(client=dorothy).member.id],
        )
        self.assertIn("daughter", [Relationship.objects.get(client=dorothy).nature])
        self.assertEqual(
            Relationship.objects.filter(
                client=dorothy,
                type__contains=Relationship.REFERENT,
            )
            .all()
            .count(),
            0,
        )

        marie = Client.objects.get(member__mid=93)
        marion = Member.objects.get(rid=865)
        self.assertEqual(
            Relationship.objects.filter(
                client=marie,
                type__contains=Relationship.REFERENT,
            )
            .all()
            .count(),
            1,
        )
        referencing = Relationship.objects.filter(
            client=marie,
            type__contains=Relationship.REFERENT,
        ).first()
        self.assertEqual(referencing.member, marion)
        self.assertEqual(referencing.member.work_information, "CLSC St-Louis du Parc")
        self.assertEqual(
            referencing.extra_fields["referral_reason"],
            "Low mobility",
        )


class ImportMemberOrdersTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ["sample_data.json"]

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """

        call_command("importclients", file="mock_clients.csv")
        call_command("importorders", file="mock_orders.csv")

    def test_import_orders(self):
        dorothy = Client.objects.get(member__mid=94)
        orders = Order.objects.get_billable_orders_client(11, 2016, dorothy)
        # Dorothy must have one billable order
        self.assertEqual(orders.count(), 1)
        self.assertEqual(orders.first().status, "D")
        self.assertEqual(orders.first().delivery_date, date(2016, 11, 11))

    def test_import_order_items(self):
        dorothy = Client.objects.get(member__mid=94)
        order = Order.objects.get_billable_orders_client(11, 2016, dorothy).first()
        items = order.orders
        main_dish = items.get(component_group="main_dish")
        self.assertEqual(main_dish.total_quantity, 2)
        self.assertEqual(main_dish.size, "L")
        diabetic_dessert = items.get(component_group="diabetic")
        self.assertEqual(diabetic_dessert.total_quantity, 1)
        dessert = items.get(component_group="dessert")
        self.assertEqual(dessert.total_quantity, 1)



import csv

from django.core.management.base import BaseCommand

from souschef.member.models import (
    CELL,
    EMAIL,
    HOME,
    WORK,
    Address,
    Contact,
    Member,
)


class Command(BaseCommand):
    help = "Data: import clients from given csv file."

    ROW_RID = 0
    ROW_ADDRESS1 = 3
    ROW_APARTMENT = 4
    ROW_CITY = 5
    ROW_POSTAL_CODE = 6
    ROW_HOME_PHONE = 7
    ROW_CELL_PHONE = 8
    ROW_WORK_PHONE = 9
    ROW_EXT = 10
    ROW_EMAIL = 11

    def add_arguments(self, parser):
        parser.add_argument(
            "--file",
            default=False,
            help="Import mock data instead of actual data",
        )

    def handle(self, *args, **options):
        file = "mock_contactaddresses.csv" if options["file"] else "contact_address.csv"

        with open(file) as f:
            reader = csv.reader(f, delimiter=";")
            for row in reader:
                try:
                    member = Member.objects.get(rid=row[self.ROW_RID])

                    address = Address.objects.create(
                        street=row[self.ROW_ADDRESS1],
                        apartment=row[self.ROW_APARTMENT],
                        city=row[self.ROW_CITY],
                        postal_code=row[self.ROW_POSTAL_CODE],
                    )

                    member.address = address
                    member.save()

                    contacts = Contact.objects.filter(member=member)
                    contacts.delete()

                    if (
                        row[self.ROW_HOME_PHONE] != ""
                        and len(row[self.ROW_HOME_PHONE]) >= 7
                    ):
                        Contact.objects.create(
                            type=HOME, value=row[self.ROW_HOME_PHONE], member=member
                        )
                    if (
                        row[self.ROW_CELL_PHONE] != ""
                        and len(row[self.ROW_CELL_PHONE]) >= 7
                    ):
                        Contact.objects.create(
                            type=CELL, value=row[self.ROW_CELL_PHONE], member=member
                        )

                    if (
                        row[self.ROW_WORK_PHONE] != ""
                        and len(row[self.ROW_WORK_PHONE]) >= 7
                    ):
                        work = row[self.ROW_WORK_PHONE]
                        if row[self.ROW_EXT] != "":
                            work += "  #" + row[self.ROW_EXT]

                        Contact.objects.create(type=WORK, value=work, member=member)
                    if row[self.ROW_EMAIL] != "":
                        Contact.objects.create(
                            type=EMAIL, value=row[self.ROW_EMAIL], member=member
                        )

                except Member.DoesNotExist:
                    self.stdout.write(self.style.WARNING("Non existing member"))



# Generated by Django 1.10.6 on 2017-04-10 21:36

import annoying.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0029_member_address_fk_to_1to1"),
    ]

    operations = [
        migrations.CreateModel(
            name="DeliveryHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(verbose_name="date of the delivery")),
                (
                    "vehicle",
                    models.CharField(
                        choices=[
                            ("cycling", "Cycling"),
                            ("walking", "Walking"),
                            ("driving", "Driving"),
                        ],
                        max_length=20,
                        verbose_name="vehicle",
                    ),
                ),
                (
                    "client_id_sequence",
                    annoying.fields.JSONField(
                        default=[],
                        verbose_name="IDs of clients on this route (as a JSON list)",
                    ),
                ),
                (
                    "comments",
                    models.TextField(blank=True, null=True, verbose_name="comments"),
                ),
            ],
            options={
                "verbose_name": "Delivery History",
                "ordering": ["route", "-date"],
                "verbose_name_plural": "Delivery Histories",
            },
        ),
        migrations.AlterField(
            model_name="route",
            name="client_id_sequence",
            field=annoying.fields.JSONField(
                default=[], verbose_name="IDs of clients on this route (as a JSON list)"
            ),
        ),
        migrations.AddField(
            model_name="deliveryhistory",
            name="route",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="delivery_histories",
                to="member.Route",
                verbose_name="route",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="deliveryhistory",
            unique_together={("route", "date")},
        ),
    ]



# Generated by Django 1.10 on 2017-01-10 19:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0024_auto_20161227_1819"),
    ]

    operations = [
        migrations.AddField(
            model_name="route",
            name="vehicle",
            field=models.CharField(
                choices=[
                    ("cycling", "Cycling"),
                    ("walking", "Walking"),
                    ("driving", "Driving"),
                ],
                default="cycling",
                max_length=20,
                verbose_name="default vehicle",
            ),
        ),
    ]



# Generated by Django 1.10 on 2016-12-20 14:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0022_auto_20161215_1936"),
    ]

    operations = [
        migrations.AlterField(
            model_name="client",
            name="billing_payment_type",
            field=models.CharField(
                blank=True,
                choices=[
                    (" ", "----"),
                    ("3rd", "3rd Party"),
                    ("credit", "Credit card"),
                    ("eft", "EFT"),
                ],
                max_length=10,
                null=True,
                verbose_name="Payment Type",
            ),
        ),
        migrations.AlterField(
            model_name="client",
            name="route",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="member.Route",
                verbose_name="route",
            ),
        ),
    ]



# Generated by Django 1.10.6 on 2017-04-04 19:56

import django.db.models.deletion
from django.db import migrations, models


def separate_shared_address_instances(apps, schema_editor):
    """
    Before changing ForeignKey to OneToOneField, make sure that members
    use different address instances.
    """
    Member = apps.get_model("member", "Member")
    address_ids = set()
    for m in Member.objects.select_related("address").all():
        a = m.address
        if a is None:
            continue
        if a.pk not in address_ids:
            address_ids.add(a.pk)
            continue
        else:  # a.pk in address_ids
            print(
                f"Member #{m.pk} ({m.firstname} {m.lastname}) shares the Address "
                f"#{a.pk}. I'm copying this Address instance and relinking the member "
                "to it..."
            )
            # Copy this object in DB
            a.pk = None
            a.save()
            assert a.pk not in address_ids
            address_ids.add(a.pk)
            m.address = a
            m.save(update_fields=["address"])


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0028_change_linked_scheduled_status_relationship"),
    ]

    operations = [
        migrations.RunPython(
            separate_shared_address_instances, reverse_code=migrations.RunPython.noop
        ),
        migrations.AlterField(
            model_name="member",
            name="address",
            field=models.OneToOneField(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="member.Address",
                verbose_name="address",
            ),
        ),
    ]



# Generated by Django 1.10.5 on 2017-01-26 16:56

import django.db.models.deletion
from django.db import migrations, models


def migrate_emergency_contacts(apps, schema_editor):
    # set NotePriority initial
    Client = apps.get_model("member", "Client")
    EmergencyContact = apps.get_model("member", "EmergencyContact")
    for client in Client.objects.all():
        if client.emergency_contact:
            EmergencyContact.objects.create(
                client=client,
                member=client.emergency_contact,
                relationship=client.emergency_contact_relationship,
            )


def reverse_emergency_contacts(apps, schema_editor):
    # set NotePriority initial
    Client = apps.get_model("member", "Client")
    EmergencyContact = apps.get_model("member", "EmergencyContact")
    for client in Client.objects.all():
        if client.emergencycontact_set.exists():
            # Use as client emergency contact the first item of
            # relationship
            emergency_contact = client.emergencycontact_set.first()
            # reversing info
            client.emergency_contact = emergency_contact.member
            client.emergency_contact_relationship = emergency_contact.relationship
            client.save()
            # removing info
            EmergencyContact.objects.filter(client=client).delete()


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0025_route_vehicle"),
    ]

    operations = [
        migrations.CreateModel(
            name="EmergencyContact",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "relationship",
                    models.CharField(
                        blank=True,
                        max_length=100,
                        null=True,
                        verbose_name="relationship",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "emergency contacts",
            },
        ),
        migrations.AddField(
            model_name="emergencycontact",
            name="client",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Client",
                verbose_name="client",
            ),
        ),
        migrations.AddField(
            model_name="emergencycontact",
            name="member",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Member",
                verbose_name="member",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="emergencycontact",
            unique_together={("client", "member")},
        ),
        migrations.AddField(
            model_name="client",
            name="emergency_contacts",
            field=models.ManyToManyField(
                related_name="emergency_contacts_of",
                through="member.EmergencyContact",
                to="member.Member",
                verbose_name="emergency contacts",
            ),
        ),
        migrations.RunPython(migrate_emergency_contacts, reverse_emergency_contacts),
        migrations.RemoveField(
            model_name="client",
            name="emergency_contact",
        ),
        migrations.RemoveField(
            model_name="client",
            name="emergency_contact_relationship",
        ),
    ]



# Generated by Django 1.10 on 2017-01-16 14:41

import django.db.models.deletion
from django.db import migrations, models


def set_default_data(apps, schema_editor):
    # set for each product the correct language
    NoteCategory = apps.get_model("note", "NoteCategory")
    Note = apps.get_model("note", "Note")

    default_category = NoteCategory.objects.get(name="Comments (no actions needed)")

    for note in Note.objects.all():
        note.category = default_category
        note.save()


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0007_notecategory"),
    ]

    operations = [
        migrations.AddField(
            model_name="note",
            name="category",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="notes",
                to="note.NoteCategory",
                verbose_name="Category",
            ),
        ),
        migrations.RunPython(set_default_data),
    ]



import collections
from datetime import datetime

from django.contrib.auth.mixins import (
    LoginRequiredMixin,
    PermissionRequiredMixin,
)
from django.contrib.auth.views import login
from django.db.models import Prefetch
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy
from django.views.generic import TemplateView

from souschef.member.models import (
    DAYS_OF_WEEK,
    Client,
    Client_option,
    Route,
)
from souschef.order.models import Order


class HomeView(LoginRequiredMixin, PermissionRequiredMixin, TemplateView):
    permission_required = "sous_chef.read"
    template_name = "pages/home.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        today = datetime.today()
        active_clients = Client.active.all().count()
        pending_clients = Client.pending.all().count()
        birthday_clients = Client.birthday_contact.get_birthday_boys_and_girls()
        billable_orders = Order.objects.get_billable_orders(
            today.year, today.month
        ).count()
        billable_orders_year = Order.objects.filter(
            status="D", delivery_date__year=datetime.today().year
        ).count()
        route_table = self.calculate_route_table()
        context["active_clients"] = active_clients
        context["pending_clients"] = pending_clients
        context["birthday"] = birthday_clients
        context["billable_orders_month"] = billable_orders
        context["billable_orders_year"] = billable_orders_year
        context["routes"] = route_table
        context["total_scheduled_by_day"] = self.calculate_total_scheduled_by_day(
            route_table
        )
        context["total_episodic_by_day"] = self.calculate_total_episodic_by_day(
            route_table
        )
        return context

    def calculate_route_table(self):
        routes = (
            Route.objects.prefetch_related(
                Prefetch(
                    "client_set",
                    to_attr="selected_clients",
                    queryset=Client.objects.filter(
                        status__in=(Client.ACTIVE, Client.PAUSED, Client.PENDING)
                    )
                    .prefetch_related(
                        Prefetch(
                            "client_option_set",
                            queryset=Client_option.objects.select_related(
                                "option", "client"
                            )
                            .filter(option__name="meals_schedule")
                            .only("value", "option__name", "client", "client__route"),
                        )
                    )
                    .only("route", "meal_default_week", "delivery_type"),
                )
            )
            .order_by("name")
            .only("name")
        )

        route_table = []
        for route in routes:
            episodic_defaults = collections.defaultdict(int)
            schedules = collections.defaultdict(int)
            for client in route.selected_clients:
                # meals_schedule are only for ongoing clients
                meals_schedule = dict(client.meals_schedule)

                # For each day, if there's a schedule, count schedule.
                for day, _ in DAYS_OF_WEEK:
                    if day in meals_schedule:
                        schedules[day] += meals_schedule[day].get("main_dish") or 0

                if client.delivery_type == Client.EPISODIC_DELIVERY:
                    meals_default = dict(client.meals_default)
                    for day, _ in DAYS_OF_WEEK:
                        episodic_defaults[day] += (
                            meals_default[day].get(
                                "main_dish",
                            )
                            or 0
                        )

            route_table.append((route.name, episodic_defaults, schedules))
        return route_table

    def calculate_total_scheduled_by_day(self, route_table):
        total_scheduled_by_day = {}
        for day, _ in DAYS_OF_WEEK:
            total_scheduled_by_day[day] = 0
            for route_entry in route_table:
                total_scheduled_by_day[day] += route_entry[2][day]
        return total_scheduled_by_day

    def calculate_total_episodic_by_day(self, route_table):
        total_episodic_by_day = {}
        for day, _ in DAYS_OF_WEEK:
            total_episodic_by_day[day] = 0
            for route_entry in route_table:
                total_episodic_by_day[day] += route_entry[1][day]
        return total_episodic_by_day


def custom_login(request):
    if request.user.is_authenticated:
        # Redirect to home if already logged in.
        return HttpResponseRedirect(reverse_lazy("page:home"))
    else:
        return login(request)



from django.db.models.signals import post_delete
from django.dispatch import receiver

from ..models import ClientScheduledStatus


@receiver(
    post_delete,
    sender=ClientScheduledStatus,
    dispatch_uid="post_delete.ensure_pair_remove",
)
def ensure_pair_remove(sender, instance, **kwargs):
    if instance.get_pair:
        ClientScheduledStatus.objects.get(pk=instance.get_pair.pk).delete()



import os
import time

from django.conf import settings


def clean_old_pdf_files():
    now = time.time()
    a_year = 356 * 24 * 60 * 60
    for filename in os.listdir(settings.GENERATED_DOCS_DIR):
        file_path = os.path.join(settings.GENERATED_DOCS_DIR, filename)
        if (
            ".pdf" in filename
            and os.path.getmtime(file_path) < now - a_year
            and os.path.isfile(file_path)
        ):
            os.remove(file_path)



# Generated by Django 1.10.5 on 2017-03-22 16:31

import django.db.models.deletion
from django.db import migrations, models


def migrate_linked_scheduled_status(apps, schema_editor):
    ClientScheduledStatus = apps.get_model("member", "ClientScheduledStatus")

    for css in ClientScheduledStatus.objects.filter(
        linked_scheduled_status__isnull=False
    ):
        css.pair = css.linked_scheduled_status
        # remove linked_scheduled_status relationship
        css.linked_scheduled_status = None
        css.save()


def reverse_linked_scheduled_status(apps, schema_editor):
    ClientScheduledStatus = apps.get_model("member", "ClientScheduledStatus")

    for css in ClientScheduledStatus.objects.filter(pair__isnull=False):
        # make exactly the opposite of 'migrate_linked_scheduled_status'
        css.linked_scheduled_status = css.pair
        # remove pair relationship
        css.pair = None
        css.save()


class Migration(migrations.Migration):
    """
    Change linked_scheduled_status relationship to OneToOneField, because
    the relationship exactly as a 'pair'
    """

    dependencies = [
        ("member", "0027_auto_20170313_1442"),
    ]

    operations = [
        migrations.AddField(
            model_name="clientscheduledstatus",
            name="pair",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="my_pair",
                to="member.ClientScheduledStatus",
                verbose_name="Client Scheduled Status Pair",
            ),
        ),
        migrations.RunPython(
            migrate_linked_scheduled_status, reverse_linked_scheduled_status
        ),
        migrations.RemoveField(
            model_name="clientscheduledstatus",
            name="linked_scheduled_status",
        ),
    ]



from datetime import datetime

from django.contrib.admin.models import (
    ADDITION,
    LogEntry,
)
from django.core.management.base import BaseCommand

from souschef.order.models import (
    ORDER_STATUS_DELIVERED,
    ORDER_STATUS_ORDERED,
    Order,
)


class Command(BaseCommand):
    help = "Set status to Delivered for all orders that have the status\
            Ordered for the specified delivery date."

    def add_arguments(self, parser):
        parser.add_argument(
            "delivery_date",
            help="The date must be in the format YYYY-MM-DD",
        )

    def handle(self, *args, **options):
        delivery_date = datetime.strptime(options["delivery_date"], "%Y-%m-%d").date()

        numorders = Order.objects.filter(
            status=ORDER_STATUS_ORDERED, delivery_date=delivery_date
        ).update(status=ORDER_STATUS_DELIVERED)

        # Log the execution
        LogEntry.objects.log_action(
            user_id=1,
            content_type_id=1,
            object_id="",
            object_repr="Status set to delivered for orders on"
            + str(delivery_date.strftime("%Y-%m-%d %H:%M")),
            action_flag=ADDITION,
        )
        print(
            f"Status set to Delivered for {numorders} orders whose "
            f"delivery date is {delivery_date}."
        )



import random

import factory
from django.contrib.auth.models import User

from souschef.member.factories import ClientFactory
from souschef.note.models import (
    Note,
    NoteCategory,
    NotePriority,
)


class UserFactory(factory.DjangoModelFactory):
    class Meta:
        model = User

    username = factory.Sequence(lambda x: f"user{x}")


class NoteFactory(factory.DjangoModelFactory):
    class Meta:
        model = Note

    note = factory.Faker("sentence")
    author = factory.SubFactory(UserFactory)
    client = factory.SubFactory(ClientFactory)
    priority = factory.LazyAttribute(
        lambda x: random.choice(NotePriority.objects.all())
    )
    category = factory.LazyAttribute(
        lambda x: random.choice(NoteCategory.objects.all())
    )



# Generated by Django 1.11.29 on 2021-07-23 14:25

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0012_note_is_deleted"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="note",
            options={
                "ordering": ("-date_modified", "note"),
                "verbose_name_plural": "Notes",
            },
        ),
        migrations.RenameField(
            model_name="note",
            old_name="date",
            new_name="date_created",
        ),
        migrations.AddField(
            model_name="note",
            name="date_modified",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Date Modified"
            ),
        ),
        migrations.RunSQL(
            ["UPDATE note_note SET date_modified = date_created"],
            "",
        ),
    ]



import csv

from django.core.management.base import BaseCommand

from souschef.member.models import (
    Client,
    Member,
)
from souschef.order.models import Order


class Command(BaseCommand):
    help = "Data: import clients relationships from given csv file."

    ROW_MID = 0
    ROW_DATE = 1
    ROW_STATUS = 2
    ROW_MAIN_DISH_QUANTITY = 3
    ROW_SIZE = 4
    ROW_FRUIT_SALAD = 5
    ROW_GREEN_SALAD = 6
    ROW_DIABETIC_DESSERT = 7
    ROW_DESSERT = 8
    ROW_PUDDING = 9

    def add_arguments(self, parser):
        parser.add_argument(
            "--file",
            default=False,
            help="Import mock data instead of actual data",
        )

    def handle(self, *args, **options):
        file = "mock_orders.csv" if options["file"] else "clients_orders.csv"

        with open(file) as f:
            reader = csv.reader(f, delimiter=";")
            for row in reader:
                try:
                    member = Member.objects.get(mid=row[self.ROW_MID])
                    client = Client.objects.get(member=member)
                    delivery_date = row[self.ROW_DATE]
                    prices = Order.objects.get_client_prices(client)
                    items = {
                        "main_dish_default_quantity": int(
                            row[self.ROW_MAIN_DISH_QUANTITY]
                        ),
                        "dessert_default_quantity": int(row[self.ROW_DESSERT]),
                        "diabetic_default_quantity": int(
                            row[self.ROW_DIABETIC_DESSERT]
                        ),
                        "fruit_salad_default_quantity": int(row[self.ROW_FRUIT_SALAD]),
                        "green_salad_default_quantity": int(row[self.ROW_GREEN_SALAD]),
                        "pudding_default_quantity": int(row[self.ROW_PUDDING]),
                        "compote_default_quantity": 0,
                        "size_default": row[self.ROW_SIZE],
                    }

                    order = Order.objects.create_order(
                        delivery_date, client, items, prices
                    )

                    order.status = row[self.ROW_STATUS]
                    order.save()

                except Member.DoesNotExist:
                    self.stdout.write(self.style.WARNING("Non existing member"))
                except Client.DoesNotExist:
                    self.stdout.write(self.style.WARNING("Non existing client"))



# Generated by Django 1.10.5 on 2017-08-16 12:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0033_auto_20170801_1607"),
    ]

    operations = [
        migrations.AlterField(
            model_name="member",
            name="mid",
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="member",
            name="rid",
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 1.10.5 on 2017-03-13 18:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0006_auto_20160826_0007"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="component",
            options={"verbose_name_plural": "components (Dishes and recipes)"},
        ),
        migrations.AlterModelOptions(
            name="restricted_item",
            options={
                "verbose_name_plural": "restricted items (Restriction categories)"
            },
        ),
        migrations.AlterField(
            model_name="component",
            name="component_group",
            field=models.CharField(
                choices=[
                    ("main_dish", "Main Dish"),
                    ("dessert", "Dessert"),
                    ("diabetic", "Diabetic Dessert"),
                    ("fruit_salad", "Fruit Salad"),
                    ("green_salad", "Green Salad"),
                    ("pudding", "Pudding"),
                    ("compote", "Compote"),
                    ("sides", "Sides"),
                ],
                max_length=100,
                verbose_name="component group",
            ),
        ),
    ]
